@@ -3,9 +3,9 @@ import glob
 import json
 import os
 from dataclasses import dataclass, field, fields
-from enum import Enum
+from enum import Enum, StrEnum
 from pathlib import Path
-from typing import Any, Optional, Union
+from typing import Any, Optional, Sequence, Union
 
 import shtab
 
@@ -20,6 +20,19 @@ GLOBAL_CONFIG_PATH = os.path.join(
 CHECK_OPTIONS = ["config"]
 
 
+class QueryInclude(StrEnum):
+    path = "path"
+    document = "document"
+
+    def to_header(self) -> str:
+        """
+        Make the string into a nice-looking format for printing in the terminal.
+        """
+        if self.value == "document":
+            return f"{self.value.capitalize()}:\n"
+        return f"{self.value.capitalize()}: "
+
+
 class CliAction(Enum):
     vectorise = "vectorise"
     query = "query"
@@ -58,6 +71,10 @@ class Config:
     reranker_params: dict[str, Any] = field(default_factory=dict)
     check_item: Optional[str] = None
     use_absolute_path: bool = False
+    include: list[QueryInclude] = field(
+        default_factory=lambda: [QueryInclude.path, QueryInclude.document]
+    )
+    hnsw: dict[str, str | int] = field(default_factory=dict)
 
     @classmethod
     async def import_from(cls, config_dict: dict[str, Any]) -> "Config":
@@ -88,6 +105,7 @@ class Config:
                 "reranker": config_dict.get("reranker", None),
                 "reranker_params": config_dict.get("reranker_params", {}),
                 "db_settings": config_dict.get("db_settings", None),
+                "hnsw": config_dict.get("hnsw", {}),
             }
         )
 
@@ -96,15 +114,24 @@ class Config:
         final_config = {}
         default_config = Config()
         for merged_field in fields(self):
-            final_config[merged_field.name] = getattr(other, merged_field.name)
-            if not final_config[merged_field.name] or final_config[
-                merged_field.name
-            ] == getattr(default_config, merged_field.name):
-                final_config[merged_field.name] = getattr(self, merged_field.name)
+            field_name = merged_field.name
+
+            other_val = getattr(other, field_name)
+            self_val = getattr(self, field_name)
+            if isinstance(other_val, dict) and isinstance(self_val, dict):
+                final_config[field_name] = {}
+                final_config[field_name].update(self_val)
+                final_config[field_name].update(other_val)
+            else:
+                final_config[field_name] = other_val
+                if not final_config[field_name] or final_config[field_name] == getattr(
+                    default_config, field_name
+                ):
+                    final_config[field_name] = self_val
         return Config(**final_config)
 
 
-async def cli_arg_parser():
+def get_cli_parser():
     shared_parser = argparse.ArgumentParser(add_help=False)
     chunkinng_parser = argparse.ArgumentParser(add_help=False)
     chunkinng_parser.add_argument(
@@ -159,7 +186,7 @@ async def cli_arg_parser():
         help="Vectorise and send documents to chromadb.",
     )
     vectorise_parser.add_argument(
-        "file_paths", nargs="+", help="Paths to files to be vectorised."
+        "file_paths", nargs="*", help="Paths to files to be vectorised."
     ).complete = shtab.FILE
     vectorise_parser.add_argument(
         "--recursive",
@@ -197,6 +224,13 @@ async def cli_arg_parser():
         action="store_true",
         help="Use absolute path when returning the retrieval results.",
     )
+    query_parser.add_argument(
+        "--include",
+        choices=list(i.value for i in QueryInclude),
+        nargs="+",
+        help="What to include in the final output.",
+        default=["path", "document"],
+    )
 
     subparsers.add_parser("drop", parents=[shared_parser], help="Remove a collection.")
 
@@ -238,8 +272,12 @@ async def cli_arg_parser():
         parents=[shared_parser],
         help="Remove empty collections in the database.",
     )
+    return main_parser
 
-    main_args = main_parser.parse_args()
+
+async def parse_cli_args(args: Optional[Sequence[str]] = None):
+    main_parser = get_cli_parser()
+    main_args = main_parser.parse_args(args)
     if main_args.action is None:
         main_args = main_parser.parse_args(["--help"])
 
@@ -252,6 +290,7 @@ async def cli_arg_parser():
     overlap_ratio = 0.2
     query_multiplier = -1
     query_exclude = []
+    query_include = ["path", "document"]
     check_item = None
     absolute = False
     match main_args.action:
@@ -267,6 +306,7 @@ async def cli_arg_parser():
             query_multiplier = main_args.multiplier
             query_exclude = main_args.exclude
             absolute = main_args.absolute
+            query_include = main_args.include
         case "check":
             check_item = main_args.check_item
         case "init":
@@ -287,6 +327,7 @@ async def cli_arg_parser():
         query_exclude=query_exclude,
         check_item=check_item,
         use_absolute_path=absolute,
+        include=[QueryInclude(i) for i in query_include],
     )
 
 
@@ -330,6 +371,34 @@ async def find_project_config_dir(start_from: PathLike = "."):
         current_dir = parent.resolve()
 
 
+def find_project_root(
+    start_from: PathLike, root_anchor: PathLike = ".vectorcode"
+) -> str | None:
+    start_from = Path(start_from)
+    if os.path.isfile(start_from):
+        start_from = start_from.parent
+
+    while start_from:
+        if (start_from / Path(root_anchor)).is_dir():
+            return str(start_from.absolute())
+        if start_from == start_from.parent:
+            return
+        start_from = start_from.parent
+
+
+async def get_project_config(project_root: PathLike) -> Config:
+    """
+    Load config file for `project_root`.
+    Fallback to global config, and then default config.
+    """
+    if not os.path.isabs(project_root):
+        project_root = os.path.abspath(project_root)
+    local_config_path = os.path.join(project_root, ".vectorcode", "config.json")
+    if os.path.isfile(os.path.join(project_root, ".vectorcode", "config.json")):
+        return await load_config_file(local_config_path)
+    return await load_config_file()
+
+
 def expand_path(path: PathLike, absolute: bool = False) -> PathLike:
     expanded = os.path.expanduser(os.path.expandvars(path))
     if absolute:
